from torchvision import datasets, transforms
import torch
from PIL import Image
import streamlit as st
import numpy as np


class FeatureExtractor:
    output_embedding = None

    def __init__(self, model_name="resnet50"):
        print("Initializing FeatureExtractor Instance...")
        self.model_name = "resnet50"
        self.transformer = transforms.Compose([
            transforms.Resize((100, 100)),
            transforms.ToTensor()
        ])

        # fetch pretrained model
        self.model = torch.hub.load('pytorch/vision:v0.10.0', self.model_name, pretrained=True)

        # Select the desired layer
        self.layer = self.model._modules.get('avgpool')

        # attach hook to the penulimate layer
        _ = self.layer.register_forward_hook(FeatureExtractor.copy_embeddings)

        self.model.eval()  # Inference mode

    @staticmethod
    def copy_embeddings(m, i, o):
        """Copy embeddings from the penultimate layer.
        """
        print("Flattening the embedding")
        FeatureExtractor.output_embedding = o[:, :, 0, 0].detach().numpy().tolist()

    def generate_img_embedding(self, pil_img):
        # img = Image.open(img_path)
        pil_img = pil_img.convert('RGB')
        print("Creating Image Tensor")
        img_tensor = self.transformer(pil_img)
        print(img_tensor[None, :].shape)
        # img_tensor_dataset = torch.utils.data.TensorDataset(img_tensor, torch.Tensor(np.array([0])))
        # dloader = torch.utils.data.DataLoader(img_tensor_dataset, shuffle=False)
        print("Creating Image Embedding")
        # for X,_ in dloader:
            # _ = self.model(X)
        _ = self.model(img_tensor[None, :])
        return FeatureExtractor.output_embedding
